﻿using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input.Touch;
using System;
using System.Collections.Generic;

namespace CloudGame
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class CloudGame : Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Game World

        #region Clouds and Sprites

        interface ISprite
        {
            void Draw(CloudGame game);
            void Update(CloudGame game);
        }

        class Cloud : CloudGame.ISprite
        {
            public Texture2D CloudTexture;
            public Vector2 CloudPosition;
            public Vector2 CloudSpeed;
            public bool Burst = false;
            public SoundEffect CloudPopSound;

            public void Draw(CloudGame game)
            {
                if (!Burst)
                    game.spriteBatch.Draw(CloudTexture, CloudPosition, Color.White);
            }

            public void Update(CloudGame game)
            {
                if (Burst) return;

                CloudPosition += CloudSpeed;

                if (CloudPosition.X > game.GraphicsDevice.Viewport.Width)
                    CloudPosition.X = -CloudTexture.Width;

                foreach (TouchLocation touch in game.Touches)
                {
                    if (touch.State == TouchLocationState.Pressed)
                    {
                        if (CloudContains(touch.Position))
                        {
                            CloudPopSound.Play();
                            Burst = true;
                            return;
                        }
                    }
                }
            }

            public bool CloudContains(Vector2 pos)
            {
                if (pos.X < CloudPosition.X) return false;
                if (pos.X > (CloudPosition.X + CloudTexture.Width)) return false;
                if (pos.Y < CloudPosition.Y) return false;
                if (pos.Y > (CloudPosition.Y + CloudTexture.Height)) return false;
                return true;
            }

            public Cloud(Texture2D inTexture, Vector2 inPosition, Vector2 inSpeed, SoundEffect inPop)
            {
                CloudTexture = inTexture;
                CloudPosition = inPosition;
                CloudSpeed = inSpeed;
                CloudPopSound = inPop;
            }
        }


        #endregion


        List<ISprite> gameSprites = new List<ISprite>();

        Random rand = new Random();

        int noOfSprites = 100;


        SoundEffect popSound;


        public CloudGame()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            Texture2D cloudTexture = Content.Load<Texture2D>("Cloud");

            SoundEffect cloudPop = Content.Load<SoundEffect>("Pop");

            for (int i = 0; i < noOfSprites; i++)
            {
                Vector2 position =
                    new Vector2(rand.Next(GraphicsDevice.Viewport.Width),
                                rand.Next(GraphicsDevice.Viewport.Height));

                // Parallax scrolling of clouds
                Vector2 speed = new Vector2(i / 6f, 0);

                Cloud c = new Cloud(cloudTexture, position, speed, cloudPop);

                gameSprites.Add(c);
            }
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        TouchCollection Touches;

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            Touches = TouchPanel.GetState();

            foreach (ISprite sprite in gameSprites)
                sprite.Update(this);

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin();

            foreach (ISprite sprite in gameSprites)
                sprite.Draw(this);

            spriteBatch.End();
            base.Draw(gameTime);
        }
    }
}
